/*
 This extension was created by Francis Grizzly Smit. 
 This software may be used and distributed
 according to the terms of the GNU General Public License version 3.0 or above.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 details.

 Copyright (C) Francis Grizzly Smit
 Note: I borrowed heavly from Albert Palacios' SystemMenu extension
*/

//const Tweener = imports.ui.tweener;
//const Me = imports.misc.extensionUtils.getCurrentExtension();
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Lang = imports.lang;
const Shell = imports.gi.Shell;
const St = imports.gi.St;
const Util = imports.misc.util;

const Main = imports.ui.main;
const PanelMenu = imports.ui.panelMenu;
const PopupMenu = imports.ui.popupMenu;
const Panel = imports.ui.panel;
const Tweener = imports.ui.tweener;
const GnomeSession = imports.misc.gnomeSession;

const guuid = 'hplip-menu'
const Gettext = imports.gettext.domain(guuid);
const _ = Gettext.gettext;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;


//let text, button;

//const _ = function(s){ return s; };

let settingsJSON,settings,settingsID;
let icon_name = null;

let extension;

let cmds = [
{ type: "command",	text: _("About This Computer"),	action: ['gnome-control-center','info-overview']		},
{ type: "command",	text: _("User Accounts"),	action: ['gnome-control-center','user-accounts']		},
{ type: "separator" },
{ type: "command",	text: _("System Printers..."),	action: ['gnome-control-center','printers']		},
{ type: "command",	text: _("Additional Printer Settings..."),	action: ['/usr/bin/python3','/usr/share/system-config-printer/system-config-printer.py']		},
{ type: "separator" },
{ type: "desktop",	text: _("Hp Device Manager..."),	action: 'hplip.desktop'		},
//{ type: "command",	text: _("Hp Device Manager..."),	action: ['hp-toolbox']		},
{ type: "separator" },
{ type: "command",	text: _("Settings..."),	action: ['gnome-shell-extension-prefs','hplip-menu@grizzlysmit.smit.id.au']		},
{}
];



const extensionObject = new Lang.Class({
    Name: guuid+"."+guuid,
    Extends: PanelMenu.Button,

    _init: function() {


        //settingsJSON = Convenience.getSettings();
        //settings = JSON.parse(settingsJSON.get_string("settings-json"));

        //icon_name = settings.icon;
        //*
        let icon_name;
        if (settings.icon_name) {
            icon_name = settings.icon_name;
        } else {
            icon_name = 'printer';
        };
        // */
        let gicon,icon;
        let re = /^.*\.png$/;
        let re2 = /^\/.*\.png$/;
        if (!re.test(icon_name) ){
            icon = new St.Icon({ icon_name: icon_name, 
                                         style_class: 'system-status-icon' });
        } else if (re2.test(icon_name)) {
            try {
                gicon = Gio.icon_new_for_string(icon_name);
            } catch(err) {
                gicon = false;
            }
            if (gicon) {
                icon = new St.Icon({ gicon: gicon, 
                                         style_class: 'system-status-icon'  });
            } else {
                icon_name = 'printer';
                icon = new St.Icon({ icon_name: icon_name, 
                                             style_class: 'system-status-icon' });
            }
        } else {
            gicon = Gio.icon_new_for_string(Me.path + "/icons/" + icon_name);
            icon = new St.Icon({ gicon: gicon });
        }
		let label = new St.Label({ text: "" });
		this.parent(0.0, label.text);
		this.actor.add_actor(icon);

		let item = null;
		for (x in cmds) {

			if (cmds[x].type=="command") {
				item = new PopupMenu.PopupMenuItem(_(cmds[x].text));
				item.connect('activate', Lang.bind(this,
					(function() {
						var currentAction = cmds[x].action;
						/* Save context variable for binding */
						return function() {
							Util.spawn(currentAction);
						}
					})()
				));
				this.menu.addMenuItem(item);
			};

			if (cmds[x].type=="desktop") {
				var action = cmds[x].action;

				item = new PopupMenu.PopupMenuItem(_(cmds[x].text));
				item.connect('activate', Lang.bind(this,
					(function() {
						var currentAction = action;
						/* Save context variable for binding */
						return function() {
							let def = Shell.AppSystem.get_default();
							let app = def.lookup_app(currentAction);
							app.activate();
						}
					})()
				));
				this.menu.addMenuItem(item);
			};

			if (cmds[x].type=="separator") {
				this.menu.addMenuItem(new PopupMenu.PopupSeparatorMenuItem());
			};

			
		};
		
	},

	destroy: function() {
		this.parent();
	},

});

function onSettingsChanged() {

	settingsJSON = Convenience.getSettings();
	settings = JSON.parse(settingsJSON.get_string("settings-json"));

	extension.destroy();
    icon_name = settings.icon;
	extension = new extensionObject();
	Main.panel.addToStatusArea(guuid, extension, settings.position, settings.area);
}

function init(metadata) {
	Convenience.initTranslations(guuid);
	settingsJSON = Convenience.getSettings();
}

function enable() {

	settings = JSON.parse(settingsJSON.get_string("settings-json"));
	settingsID = settingsJSON.connect("changed::settings-json", Lang.bind(this,onSettingsChanged)); 

    icon_name = settings.icon;
	extension = new extensionObject();
	Main.panel.addToStatusArea(guuid, extension, settings.position, settings.area);
}

function disable() {
	settingsJSON.disconnect(settingsID);
	extension.destroy();
}
